@@ -46,10 +46,10 @@ FindAllMarkers <- function(
   object,
   assay = NULL,
   features = NULL,
-  logfc.threshold = 0.25,
+  logfc.threshold = 0.1,
   test.use = 'wilcox',
   slot = 'data',
-  min.pct = 0.1,
+  min.pct = 0.01,
   min.diff.pct = -Inf,
   node = NULL,
   verbose = TRUE,
@@ -416,12 +416,16 @@ FindConservedMarkers <- function(
 #' expressing
 #' @param features Genes to test. Default is to use all genes
 #' @param logfc.threshold Limit testing to genes which show, on average, at least
-#' X-fold difference (log-scale) between the two groups of cells. Default is 0.25
+#' X-fold difference (log-scale) between the two groups of cells. Default is 0.1
 #' Increasing logfc.threshold speeds up the function, but can miss weaker signals.
 #' @param test.use Denotes which test to use. Available options are:
 #' \itemize{
 #'  \item{"wilcox"} : Identifies differentially expressed genes between two
-#'  groups of cells using a Wilcoxon Rank Sum test (default)
+#'  groups of cells using a Wilcoxon Rank Sum test (default); will use a fast
+#'  implementation by Presto if installed
+#'  \item{"wilcox_limma"} : Identifies differentially expressed genes between two
+#'  groups of cells using the limma implementation of the Wilcoxon Rank Sum test;
+#'  set this option to reproduce results from Seurat v4
 #'  \item{"bimod"} : Likelihood-ratio test for single cell gene expression,
 #'  (McDavid et al., Bioinformatics, 2013)
 #'  \item{"roc"} : Identifies 'markers' of gene expression using ROC analysis.
@@ -460,7 +464,7 @@ FindConservedMarkers <- function(
 #' }
 #' @param min.pct  only test genes that are detected in a minimum fraction of
 #' min.pct cells in either of the two populations. Meant to speed up the function
-#' by not testing genes that are very infrequently expressed. Default is 0.1
+#' by not testing genes that are very infrequently expressed. Default is 0.01
 #' @param min.diff.pct  only test genes that show a minimum difference in the
 #' fraction of detection between the two groups. Set to -Inf by default
 #' @param only.pos Only return positive markers (FALSE by default)
@@ -494,9 +498,9 @@ FindMarkers.default <- function(
   cells.1 = NULL,
   cells.2 = NULL,
   features = NULL,
-  logfc.threshold = 0.25,
+  logfc.threshold = 0.1,
   test.use = 'wilcox',
-  min.pct = 0.1,
+  min.pct = 0.01,
   min.diff.pct = -Inf,
   verbose = TRUE,
   only.pos = FALSE,
@@ -510,7 +514,6 @@ FindMarkers.default <- function(
   densify = FALSE,
   ...
 ) {
-  pseudocount.use <- pseudocount.use %||% 1
   ValidateCellGroups(
     object = object,
     cells.1 = cells.1,
@@ -573,18 +576,35 @@ FindMarkers.default <- function(
       latent.vars <- latent.vars[c(cells.1, cells.2), , drop = FALSE]
     }
   }
-  de.results <- PerformDE(
-    object = object,
-    cells.1 = cells.1,
-    cells.2 = cells.2,
-    features = features,
-    test.use = test.use,
-    verbose = verbose,
-    min.cells.feature = min.cells.feature,
-    latent.vars = latent.vars,
-    densify = densify,
-    ...
-  )
+  if (inherits(x = object, what = "IterableMatrix")){
+    if(test.use != "wilcox"){
+      stop("Differential expression with BPCells currently only supports the 'wilcox' method.",
+           " Please rerun with test.use = 'wilcox'")
+    }
+    data.use <- object[features, c(cells.1, cells.2), drop = FALSE]
+    groups <- c(rep("foreground", length(cells.1)), rep("background", length(cells.2)))
+    de.results <- suppressMessages(
+      BPCells::marker_features(data.use, group = groups, method = "wilcoxon")
+    )
+    de.results <- subset(de.results, de.results$foreground == "foreground")
+    de.results <- data.frame(feature = de.results$feature,
+                             p_val = de.results$p_val_raw)
+    rownames(de.results) <- de.results$feature
+    de.results$feature <- NULL
+  } else {
+    de.results <- PerformDE(
+      object = object,
+      cells.1 = cells.1,
+      cells.2 = cells.2,
+      features = features,
+      test.use = test.use,
+      verbose = verbose,
+      min.cells.feature = min.cells.feature,
+      latent.vars = latent.vars,
+      densify = densify,
+      ...
+    )
+  }
   de.results <- cbind(de.results, fc.results[rownames(x = de.results), , drop = FALSE])
   if (only.pos) {
     de.results <- de.results[de.results[, 2] > 0, , drop = FALSE]
@@ -592,7 +612,7 @@ FindMarkers.default <- function(
   if (test.use %in% DEmethods_nocorrect()) {
     de.results <- de.results[order(-de.results$power, -de.results[, 1]), ]
   } else {
-    de.results <- de.results[order(de.results$p_val, -de.results[, 1]), ]
+    de.results <- de.results[order(de.results$p_val, -abs(de.results[,colnames(fc.results)[1]])), ]
     de.results$p_val_adj = p.adjust(
       p = de.results$p_val,
       method = "bonferroni",
@@ -616,9 +636,9 @@ FindMarkers.Assay <- function(
   cells.1 = NULL,
   cells.2 = NULL,
   features = NULL,
-  logfc.threshold = 0.25,
+  logfc.threshold = 0.1,
   test.use = 'wilcox',
-  min.pct = 0.1,
+  min.pct = 0.01,
   min.diff.pct = -Inf,
   verbose = TRUE,
   only.pos = FALSE,
@@ -635,12 +655,14 @@ FindMarkers.Assay <- function(
   norm.method = NULL,
   ...
 ) {
-  pseudocount.use <- pseudocount.use %||% 1
   data.slot <- ifelse(
     test = test.use %in% DEmethods_counts(),
     yes = 'counts',
     no = slot
   )
+  if (length(x = Layers(object = object, search = slot)) > 1) {
+    stop(slot, ' layers are not joined. Please run JoinLayers')
+  }
   data.use <-  GetAssayData(object = object, slot = data.slot)
   counts <- switch(
     EXPR = data.slot,
@@ -685,6 +707,11 @@ FindMarkers.Assay <- function(
   return(de.results)
 }
 
+#' @method FindMarkers StdAssay
+#' @export
+#'
+FindMarkers.StdAssay <- FindMarkers.Assay
+
 #' @param recorrect_umi Recalculate corrected UMI counts using minimum of the median UMIs when performing DE using multiple SCT objects; default is TRUE
 #'
 #' @rdname FindMarkers
@@ -698,9 +725,9 @@ FindMarkers.SCTAssay <- function(
   cells.1 = NULL,
   cells.2 = NULL,
   features = NULL,
-  logfc.threshold = 0.25,
+  logfc.threshold = 0.1,
   test.use = 'wilcox',
-  min.pct = 0.1,
+  min.pct = 0.01,
   min.diff.pct = -Inf,
   verbose = TRUE,
   only.pos = FALSE,
@@ -717,7 +744,6 @@ FindMarkers.SCTAssay <- function(
   recorrect_umi = TRUE,
   ...
 ) {
-  pseudocount.use <- pseudocount.use %||% 1
   data.slot <- ifelse(
     test = test.use %in% DEmethods_counts(),
     yes = 'counts',
@@ -761,12 +787,14 @@ FindMarkers.SCTAssay <- function(
   )
   # Default assumes the input is log1p(corrected counts)
   default.mean.fxn <- function(x) {
-    return(log(x = rowMeans(x = expm1(x = x)) + pseudocount.use, base = base))
+    # return(log(x = rowMeans(x = expm1(x = x)) + pseudocount.use, base = base))
+    return(log(x = (rowSums(x = expm1(x = x)) + pseudocount.use)/NCOL(x), base = base))
   }
   mean.fxn <- mean.fxn %||% switch(
     EXPR = slot,
     'counts' = function(x) {
-      return(log(x = rowMeans(x = x) + pseudocount.use, base = base))
+      # return(log(x = rowMeans(x = x) + pseudocount.use, base = base))
+      return(log(x = (rowSums(x = x) + pseudocount.use)/NCOL(x), base = base))
     },
     'scale.data' = rowMeans,
     default.mean.fxn
@@ -820,9 +848,9 @@ FindMarkers.DimReduc <- function(
   cells.1 = NULL,
   cells.2 = NULL,
   features = NULL,
-  logfc.threshold = 0.25,
+  logfc.threshold = 0.1,
   test.use = "wilcox",
-  min.pct = 0.1,
+  min.pct = 0.01,
   min.diff.pct = -Inf,
   verbose = TRUE,
   only.pos = FALSE,
@@ -838,7 +866,6 @@ FindMarkers.DimReduc <- function(
   ...
 
 ) {
-  pseudocount.use <- pseudocount.use %||% 1
   if (test.use %in% DEmethods_counts()) {
     stop("The following tests cannot be used for differential expression on a reduction as they assume a count model: ",
          paste(DEmethods_counts(), collapse=", "))
@@ -901,7 +928,7 @@ FindMarkers.DimReduc <- function(
     de.results$p_val_adj = p.adjust(
       p = de.results$p_val,
       method = "bonferroni",
-      n = nrow(x = object)
+      n = ncol(x = object)
     )
   }
   return(de.results)
@@ -942,9 +969,10 @@ FindMarkers.Seurat <- function(
   slot = 'data',
   reduction = NULL,
   features = NULL,
-  logfc.threshold = 0.25,
+  logfc.threshold = 0.1,
+  pseudocount.use = 1,
   test.use = "wilcox",
-  min.pct = 0.1,
+  min.pct = 0.01,
   min.diff.pct = -Inf,
   verbose = TRUE,
   only.pos = FALSE,
@@ -986,6 +1014,18 @@ FindMarkers.Seurat <- function(
     ident.2 = ident.2,
     cellnames.use = cellnames.use
   )
+  cells <- sapply(
+    X = cells,
+    FUN = intersect,
+    y = cellnames.use,
+    simplify = FALSE,
+    USE.NAMES = TRUE
+  )
+  if (!all(vapply(X = cells, FUN = length, FUN.VALUE = integer(length = 1L)))) {
+    abort(
+      message = "Cells in one or both identity groups are not present in the data requested"
+    )
+  }
   # fetch latent.vars
   if (!is.null(x = latent.vars)) {
     latent.vars <- FetchData(
@@ -1019,6 +1059,7 @@ FindMarkers.Seurat <- function(
     cells.2 = cells$cells.2,
     features = features,
     logfc.threshold = logfc.threshold,
+    pseudocount.use = pseudocount.use,
     test.use = test.use,
     min.pct = min.pct,
     min.diff.pct = min.diff.pct,
@@ -1083,6 +1124,7 @@ FoldChange.default <- function(
 #' when \code{slot} is \dQuote{\code{data}}
 #'
 #' @importFrom Matrix rowMeans
+#' @importFrom Matrix rowSums
 #' @rdname FoldChange
 #' @concept differential_expression
 #' @export
@@ -1100,15 +1142,16 @@ FoldChange.Assay <- function(
   norm.method = NULL,
   ...
 ) {
-  pseudocount.use <- pseudocount.use %||% 1
   data <- GetAssayData(object = object, slot = slot)
   # By default run as if LogNormalize is done
   log1pdata.mean.fxn <- function(x) {
-    return(log(x = rowMeans(x = expm1(x = x)) + pseudocount.use, base = base))
+    # return(log(x = rowMeans(x = expm1(x = x)) + pseudocount.use, base = base))
+    return(log(x = (rowSums(x = expm1(x = x)) + pseudocount.use)/NCOL(x), base = base))
   }
   scaledata.mean.fxn <- rowMeans
   counts.mean.fxn <- function(x) {
-    return(log(x = rowMeans(x = x) + pseudocount.use, base = base))
+    # return(log(x = rowMeans(x = x) + pseudocount.use, base = base))
+    return(log(x = (rowSums(x = x) + pseudocount.use)/NCOL(x), base = base))
   }
   if (!is.null(x = norm.method)) {
     # For anything apart from log normalization set to rowMeans
@@ -1154,7 +1197,13 @@ FoldChange.Assay <- function(
   )
 }
 
+#' @method FoldChange StdAssay
+#' @export
+#'
+FoldChange.StdAssay <- FoldChange.Assay
+
 #' @importFrom Matrix rowMeans
+#' @importFrom Matrix rowSums
 #' @rdname FoldChange
 #' @concept differential_expression
 #' @export
@@ -1174,14 +1223,16 @@ FoldChange.SCTAssay <- function(
   pseudocount.use <- pseudocount.use %||% 1
   data <- GetAssayData(object = object, slot = slot)
   default.mean.fxn <- function(x) {
-    return(log(x = rowMeans(x = expm1(x = x)) + pseudocount.use, base = base))
+    # return(log(x = rowMeans(x = expm1(x = x)) + pseudocount.use, base = base))
+    return(log(x = (rowSums(x = expm1(x = x)) + pseudocount.use)/NCOL(x), base = base))
   }
   mean.fxn <- mean.fxn %||% switch(
     EXPR = slot,
     'data' = default.mean.fxn,
     'scale.data' = rowMeans,
     'counts' = function(x) {
-      return(log(x = rowMeans(x = x) + pseudocount.use, base = base))
+      # return(log(x = rowMeans(x = x) + pseudocount.use, base = base))
+      return(log(x = (rowSums(x = x) + pseudocount.use)/NCOL(x), base = base))
     },
     default.mean.fxn
   )
@@ -1223,7 +1274,6 @@ FoldChange.DimReduc <- function(
   mean.fxn = NULL,
   ...
 ) {
-  pseudocount.use <- pseudocount.use %||% 1
   mean.fxn <- mean.fxn %||% rowMeans
   fc.name <- fc.name %||% "avg_diff"
   data <- t(x = Embeddings(object = object))
@@ -1271,7 +1321,7 @@ FoldChange.Seurat <- function(
   slot = 'data',
   reduction = NULL,
   features = NULL,
-  pseudocount.use = NULL,
+  pseudocount.use = 1,
   mean.fxn = NULL,
   base = 2,
   fc.name = NULL,
@@ -1406,7 +1456,7 @@ DEmethods_latent <- function() {
 
 # returns tests that require CheckDots
 DEmethods_checkdots <- function() {
-  c('wilcox', 'MAST', 'DESeq2')
+  c('wilcox', 'wilcox_limma', 'MAST', 'DESeq2')
 }
 
 # returns tests that do not use Bonferroni correction on the DE results
@@ -2045,6 +2095,14 @@ PerformDE <- function(
       verbose = verbose,
       ...
     ),
+    'wilcox_limma' = WilcoxDETest(
+      data.use = data.use,
+      cells.1 = cells.1,
+      cells.2 = cells.2,
+      verbose = verbose,
+      limma = TRUE,
+      ...
+    ),
     'bimod' = DiffExpTest(
       data.use = data.use,
       cells.1 = cells.1,
@@ -2120,10 +2178,12 @@ PerformDE <- function(
 #' @param assay Assay name where for SCT objects are stored; Default is 'SCT'
 #' @param verbose Print messages and progress
 #' @importFrom Matrix Matrix
+#' @importFrom SeuratObject SparseEmptyMatrix
 #' @importFrom pbapply pblapply
 #' @importFrom future.apply future_lapply
 #' @importFrom future nbrOfWorkers
 #' @importFrom sctransform correct_counts
+#' @importFrom SeuratObject JoinLayers
 #'
 #' @return Returns a Seurat object with recorrected counts and data in the SCT assay.
 #' @export
@@ -2133,8 +2193,8 @@ PerformDE <- function(
 #' @template section-future
 #' @examples
 #' data("pbmc_small")
-#' pbmc_small1 <- SCTransform(object = pbmc_small, variable.features.n = 20)
-#' pbmc_small2 <- SCTransform(object = pbmc_small, variable.features.n = 20)
+#' pbmc_small1 <- SCTransform(object = pbmc_small, variable.features.n = 20, vst.flavor="v1")
+#' pbmc_small2 <- SCTransform(object = pbmc_small, variable.features.n = 20, vst.flavor="v1")
 #' pbmc_merged <- merge(x = pbmc_small1, y = pbmc_small2)
 #' pbmc_merged <- PrepSCTFindMarkers(object = pbmc_merged)
 #' markers <- FindMarkers(
@@ -2206,6 +2266,12 @@ PrepSCTFindMarkers <- function(object, assay = "SCT", verbose = TRUE) {
          paste(umi.assay, collapse = ", ")
     )
   }
+  umi.layers <- Layers(object = object, assay = umi.assay, search = 'counts')
+  if (length(x = umi.layers) > 1) {
+    object[[umi.assay]] <- JoinLayers(
+      object = object[[umi.assay]],
+      layers = "counts", new = "counts")
+  }
   raw_umi <- GetAssayData(object = object, assay = umi.assay, slot = "counts")
   corrected_counts <- Matrix(
     nrow = nrow(x = raw_umi),
@@ -2224,7 +2290,7 @@ PrepSCTFindMarkers <- function(object, assay = "SCT", verbose = TRUE) {
   set_median_umi <- rep(min_median_umi, length(levels(x = object[[assay]])))
   names(set_median_umi) <- levels(x = object[[assay]])
   set_median_umi <- as.list(set_median_umi)
-
+  all_genes <- rownames(x = object[[assay]])
   # correct counts
   my.correct_counts <- function(model_name){
     model_genes <- rownames(x = model_pars_fit[[model_name]])
@@ -2235,7 +2301,7 @@ PrepSCTFindMarkers <- function(object, assay = "SCT", verbose = TRUE) {
         cell_attr = cell_attr[[model_name]]
       )
       cells <- rownames(x = cell_attr[[model_name]])
-      umi <- raw_umi[model_genes, cells]
+      umi <- raw_umi[all_genes, cells]
 
       umi_corrected <- correct_counts(
         x = x,
@@ -2243,14 +2309,23 @@ PrepSCTFindMarkers <- function(object, assay = "SCT", verbose = TRUE) {
         verbosity = 0,
         scale_factor = min_median_umi
       )
+      missing_features <- setdiff(x = all_genes, y = rownames(x = umi_corrected))
+      corrected_counts.list <- NULL
+      gc(verbose = FALSE)
+      empty <- SparseEmptyMatrix(nrow = length(x = missing_features), ncol = ncol(x = umi_corrected))
+      rownames(x = empty) <- missing_features
+      colnames(x = umi_corrected) <- colnames(x = umi_corrected)
+
+      umi_corrected <- rbind(umi_corrected, empty)[all_genes,]
+
       return(umi_corrected)
   }
   corrected_counts.list <- my.lapply(X = levels(x = object[[assay]]),
                                      FUN = my.correct_counts)
   names(x = corrected_counts.list) <- levels(x = object[[assay]])
-  corrected_counts <- do.call(what = MergeSparseMatrices, args = corrected_counts.list)
-  corrected_counts.list <- NULL
 
+  corrected_counts <- do.call(what = MergeSparseMatrices, args = corrected_counts.list)
+  corrected_counts <- as.sparse(x = corrected_counts)
   corrected_data <- log1p(x = corrected_counts)
   suppressWarnings({object <- SetAssayData(object = object,
                                            assay = assay,
@@ -2264,6 +2339,19 @@ PrepSCTFindMarkers <- function(object, assay = "SCT", verbose = TRUE) {
   return(object)
 }
 
+PrepSCTFindMarkers.V5 <- function(object, assay = "SCT", umi.assay = "RNA", layer = "counts", verbose = TRUE) {
+  layers <- Layers(object = object[[umi.assay]], search = layer)
+  dataset.names <- gsub(pattern = paste0(layer, "."), replacement = "", x = layers)
+  for (i in seq_along(along.with = layers)) {
+    l <- layers[i]
+    counts <- LayerData(
+      object = object[[umi.assay]],
+      layer = l
+      )
+  }
+  cells.grid <- DelayedArray::colAutoGrid(x = counts, ncol = min(length(Cells(object)), ncol(counts)))
+}
+
 # given a UMI count matrix, estimate NB theta parameter for each gene
 # and use fit of relationship with mean to assign regularized theta to each gene
 #
@@ -2368,14 +2456,18 @@ ValidateCellGroups <- function(
 # Differential expression using Wilcoxon Rank Sum
 #
 # Identifies differentially expressed genes between two groups of cells using
-# a Wilcoxon Rank Sum test. Makes use of limma::rankSumTestWithCorrelation for a
+# a Wilcoxon Rank Sum test. Makes use of presto::wilcoxauc for a more efficient
+# implementation of the wilcoxon test. If presto is not installed, or if limma
+# is requested, makes use of limma::rankSumTestWithCorrelation for a
 # more efficient implementation of the wilcoxon test. Thanks to Yunshun Chen and
-# Gordon Smyth for suggesting the limma implementation.
+# Gordon Smyth for suggesting the limma implementation. If limma is also not installed,
+# uses wilcox.test.
 #
 # @param data.use Data matrix to test
 # @param cells.1 Group 1 cells
 # @param cells.2 Group 2 cells
 # @param verbose Print a progress bar
+# @param limma If limma should be used for testing; default is FALSE
 # @param ... Extra parameters passed to wilcox.test
 #
 # @return Returns a p-value ranked matrix of putative differentially expressed
@@ -2399,6 +2491,7 @@ WilcoxDETest <- function(
   cells.1,
   cells.2,
   verbose = TRUE,
+  limma = FALSE,
   ...
 ) {
   data.use <- data.use[, c(cells.1, cells.2), drop = FALSE]
@@ -2413,40 +2506,59 @@ WilcoxDETest <- function(
     yes = FALSE,
     no = TRUE
   )
+  presto.check <- PackageCheck("presto", error = FALSE)
   limma.check <- PackageCheck("limma", error = FALSE)
-  if (limma.check[1] && overflow.check) {
-    p_val <- my.sapply(
-      X = 1:nrow(x = data.use),
-      FUN = function(x) {
-        return(min(2 * min(limma::rankSumTestWithCorrelation(index = j, statistics = data.use[x, ])), 1))
-      }
-    )
+  group.info <- data.frame(row.names = c(cells.1, cells.2))
+  group.info[cells.1, "group"] <- "Group1"
+  group.info[cells.2, "group"] <- "Group2"
+  group.info[, "group"] <- factor(x = group.info[, "group"])
+  if (presto.check[1] && (!limma)) {
+    data.use <- data.use[, rownames(group.info), drop = FALSE]
+    res <- presto::wilcoxauc(X = data.use, y = group.info[, "group"])
+    res <- res[1:(nrow(x = res)/2),]
+    p_val <- res$pval
   } else {
-    if (getOption('Seurat.limma.wilcox.msg', TRUE) && overflow.check) {
+    if (getOption('Seurat.presto.wilcox.msg', TRUE) && (!limma)) {
       message(
-        "For a more efficient implementation of the Wilcoxon Rank Sum Test,",
-        "\n(default method for FindMarkers) please install the limma package",
+        "For a (much!) faster implementation of the Wilcoxon Rank Sum Test,",
+        "\n(default method for FindMarkers) please install the presto package",
         "\n--------------------------------------------",
-        "\ninstall.packages('BiocManager')",
-        "\nBiocManager::install('limma')",
+        "\ninstall.packages('devtools')",
+        "\ndevtools::install_github('immunogenomics/presto')",
         "\n--------------------------------------------",
-        "\nAfter installation of limma, Seurat will automatically use the more ",
+        "\nAfter installation of presto, Seurat will automatically use the more ",
         "\nefficient implementation (no further action necessary).",
         "\nThis message will be shown once per session"
       )
-      options(Seurat.limma.wilcox.msg = FALSE)
+      options(Seurat.presto.wilcox.msg = FALSE)
     }
-    group.info <- data.frame(row.names = c(cells.1, cells.2))
-    group.info[cells.1, "group"] <- "Group1"
-    group.info[cells.2, "group"] <- "Group2"
-    group.info[, "group"] <- factor(x = group.info[, "group"])
-    data.use <- data.use[, rownames(x = group.info), drop = FALSE]
-    p_val <- my.sapply(
-      X = 1:nrow(x = data.use),
-      FUN = function(x) {
-        return(wilcox.test(data.use[x, ] ~ group.info[, "group"], ...)$p.value)
+    if (limma.check[1] && overflow.check) {
+      p_val <- my.sapply(
+        X = 1:nrow(x = data.use),
+        FUN = function(x) {
+          return(min(2 * min(limma::rankSumTestWithCorrelation(index = j, statistics = data.use[x, ])), 1))
+        }
+      )
+    } else {
+      if (limma && overflow.check) {
+        stop(
+          "To use the limma implementation of the Wilcoxon Rank Sum Test,
+        please install the limma package:
+        --------------------------------------------
+        install.packages('BiocManager')
+        BiocManager::install('limma')
+        --------------------------------------------"
+        )
+      } else {
+        data.use <- data.use[, rownames(x = group.info), drop = FALSE]
+        p_val <- my.sapply(
+          X = 1:nrow(x = data.use),
+          FUN = function(x) {
+            return(wilcox.test(data.use[x, ] ~ group.info[, "group"], ...)$p.value)
+          }
+        )
       }
-    )
+    }
   }
   return(data.frame(p_val, row.names = rownames(x = data.use)))
 }
